"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));
require("core-js/modules/es.promise.js");
var _forestExpress = require("forest-express");
var _httpErrors = _interopRequireDefault(require("http-errors"));
var _errors = require("./errors");
var _queryOptions = _interopRequireDefault(require("./query-options"));
var _resourceGetter = _interopRequireDefault(require("./resource-getter"));
function ownKeys(e, r) { var t = Object.keys(e); if (Object.getOwnPropertySymbols) { var o = Object.getOwnPropertySymbols(e); r && (o = o.filter(function (r) { return Object.getOwnPropertyDescriptor(e, r).enumerable; })), t.push.apply(t, o); } return t; }
function _objectSpread(e) { for (var r = 1; r < arguments.length; r++) { var t = null != arguments[r] ? arguments[r] : {}; r % 2 ? ownKeys(Object(t), !0).forEach(function (r) { (0, _defineProperty2.default)(e, r, t[r]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(e, Object.getOwnPropertyDescriptors(t)) : ownKeys(Object(t)).forEach(function (r) { Object.defineProperty(e, r, Object.getOwnPropertyDescriptor(t, r)); }); } return e; }
class ResourceUpdater {
  constructor(model, params, newRecord, user) {
    this._model = model.unscoped();
    this._params = params;
    this._newRecord = newRecord;
    this._user = user;
  }
  async perform() {
    const {
      timezone
    } = this._params;
    const scopeFilters = await _forestExpress.scopeManager.getScopeForUser(this._user, this._model.name, true);
    const queryOptions = new _queryOptions.default(this._model);
    await queryOptions.filterByIds([this._params.recordId]);
    await queryOptions.filterByConditionTree(scopeFilters, timezone);
    const record = await this._model.findOne(queryOptions.sequelizeOptions);
    if (record) {
      Object.assign(record, this._newRecord);
      try {
        await record.validate();
        await record.save();
      } catch (error) {
        throw new _errors.ErrorHTTP422(error.message);
      }
    } else {
      throw (0, _httpErrors.default)(404, `The ${this._model.name} #${this._params.recordId} does not exist.`);
    }
    try {
      return await new _resourceGetter.default(this._model, _objectSpread(_objectSpread({}, this._params), {}, {
        recordId: this._params.recordId
      }), this._user).perform();
    } catch (error) {
      if (error.statusCode === 404 && scopeFilters) {
        return record;
      }
      throw error;
    }
  }
}
module.exports = ResourceUpdater;